/**
 * Copyright Copyright 2008 Simon Andrews and Curtis Rueden
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw.IO;

import ij.ImagePlus;
import ij.ImageStack;
import ij.process.ImageProcessor;

import java.io.File;
import java.io.IOException;
import loci.formats.ImageReader;
import loci.plugins.Util;

public class BioFormatsImageLoader  {
	
	public static ImagePlus [] loadImages (File file) throws IOException {
						
		try {
			
			ImageReader reader = new ImageReader();
			reader.setId(file.getAbsolutePath());
						
			/*
			 * BioFormats image channels are a mix of separate
			 * collection channels and colour channels.
			 * 
			 * getSizeC gives you the total number of channels
			 * and getRBGChannel count tells you how many colour
			 * channels there are.  EffectiveSizeC is the total
			 * channels divided by the colour channels.
			 * 
			 */
			
			int seriesCount = reader.getEffectiveSizeC();
			
			ImagePlus[] ip = new ImagePlus[seriesCount];
			ImageStack[] stacks = new ImageStack[seriesCount];
						
			for (int s=0;s<stacks.length;s++) {
				stacks[s] = new ImageStack(reader.getSizeX(),reader.getSizeY());
			}
			
			for (int i=0;i<reader.getImageCount();i++) {
				// read image plane

				ImageProcessor [] procs = Util.openProcessors(reader, i);

				// determine channel for this plane and add to appropriate stack
				int[] zct = reader.getZCTCoords(i);
				int c = zct[1]/reader.getRGBChannelCount();

//				System.err.println("From file "+file.getName()+" and image "+i+" I got stack "+c+" with "+procs.length+" procs");
				
				/*
				 * I can use the Util.makeRGB(ImageProcessor[]) to get a single
				 * ImagePlus object from a processor array.
				 * 
				 */
				
				/*
				 * TODO: There is a problem here when using CMYK images (eg demo confocal.ics)
				 * ImageJ will correctly display this in colour, but my JImagePanel shows it
				 * as black and white.  I may need to modify a newer version of ImagePanel.
				 */

				stacks[c].addSlice("",Util.makeRGB(procs).getProcessor());
			}
			reader.close();
			// Temporary - trying to find memory leaks!
			reader = null;

			for (int s=0;s<stacks.length;s++) {
				ip[s] = new ImagePlus(file.getName(),stacks[s]);
			}
			return ip;
		} 
		catch (Exception e) {
			e.printStackTrace();
			throw new IOException(e.getLocalizedMessage());
		}
	}
		
}
